
#include <GL/glut.h>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>           /* ULONG_MAX is defined here */
#include <float.h>            /* FLT_MAX is atleast defined here */

#include <time.h>  /* for random seed */
#include "asteroids.h"

GLfloat light_diffuse[] = {1.0, 1.0, 1.0, 1.0};  /* White diffuse light. */
GLfloat light_position[] = {100.0, 100.0, 400.0, 0.0};  /* Infinite light location. */
static GLfloat blueMaterial[] = {0.0, 0.2, 1.0, 1.0},
  redMaterial[] = {0.9, 0.2, 0.2, 1.0},
  yellowMaterial[] = {0.9, 0.8, 0.3, 1.0},
  greenMaterial[] = {1.0, 0.2, 0.0, 1.0},
  whiteMaterial[] = {1.0, 1.0, 1.0, 1.0};

/* Some <math.h> files do not define M_PI... */
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#ifdef _WIN32
#define drand48() (((float) rand())/((float) RAND_MAX))
#define srand48(x) (srand((x)))
#endif

#define SPACESIZE  100

#define ROTSPEED 180           // gradi per secs
#define THRUSTACCEL 0.05    // mt al sec per sec
#define BULLETSPEED 50      // mt al sec

#define MAXAST 50
#define MAXBULLET 10
#define BULLETTIME 1500

#define MAXEXP 10

/***************************************************************/
/************************* ASTEROIDS TYPES *********************/
/***************************************************************/
class GameObj
{
 public:
	 GameObj();
	float x,y;	   // Position
	float angle;  // Direction (in degree)
	float dx,dy;  // Speed Component (mt / sec)  
	int active;
	int DL;
	int CreateDL();
 	void virtual Draw(int time)=0;
	void Update(int t);
};

class Ast : public GameObj
{
 public:
	float dangle; // Angular Speed 
	float size;
	Point3f A;    // Rotation Axis
	void Update(int time);
	void Draw(int time);
	void CreateDL();
	void Hit(Ast &a);
};

class Bullet : public GameObj
{
 public:
	int ttl;
	void Draw(int time);
	void Update(int t);
	int Hit(Ast &A);
};

class Ship : public GameObj
{
 public:
	int thrust;
	int rotating;
	Bullet B[MAXBULLET];

	void Draw(int time);
	void Init();
	void Update(int t);
	void Ship::ShootBullet(int i);
	int Ship::GetFreeBullet();
};

class Exp : public GameObj
{
public:
	int ttl;
	void Draw(int time);
	void Update(int t);
};

/***************************************************************/
/****************************** GLOBALS ************************/
/***************************************************************/

Ship S;
Ast A[MAXAST];
Exp E[MAXEXP];
int seed;

int VL=-SPACESIZE,VR=SPACESIZE,VB=-SPACESIZE,VT=SPACESIZE;
int SW=1,SH=1;


/***************************************************************/
/************************ ASTEROID STUFF ***********************/
/***************************************************************/
/************************ Asteroid STUFF ***********************/
void GameObj::Update(int t)
{
 x+=dx*t;
 y+=dy*t;
 if(x>VR) x -=(VR-VL);
 if(x<VL) x +=(VR-VL);
 if(y>VT) y -=(VT-VB);
 if(y<VB) y +=(VT-VB);
}

GameObj::GameObj()
{
	x=0;
	y=0;
	dx=0;
	dy=0;
	active=0;
}

int GetFreeAst(Ast *A)
{
	int i;
	for(i=0;i<MAXAST;i++)
		if(A[i].active==0)
		return i;
   return 0;
}

int NewAst(float size)
{
 int i;
	i=GetFreeAst(A);
	A[i].x=VL+rand()%(VR-VL);
	A[i].y=VB+rand()%(VT-VB);
	A[i].dx=((rand()%30) - 15)/1000.0;
	A[i].dy=((rand()%30) - 15)/1000.0;
	A[i].dangle=(rand()%100) - 50;
	A[i].A.x=drand48()-0.5;
	A[i].A.y=drand48()-0.5;
	A[i].A.z=drand48()-0.5;
	A[i].active=1;
	A[i].size=size;
	A[i].CreateDL();
return i;
}

void Ast::CreateDL()
{
	int level=1;
	if(size>5) level=2;
	if(size>10) level=3;

	DL=glGenLists(1);
	glNewList(DL,GL_COMPILE);
	//glutWireSphere(1,8,6);
    CreateAsteroid(level,1.8, 2,seed++);
	glEndList();
}


void Ast::Update(int t)
{
 GameObj::Update(t);
 angle+=dangle*t/1000.0;
}

void Ast::Draw(int t)
{
 glPushMatrix();
 glTranslatef(x,y,0);
 glRotatef(angle,A.x,A.y,A.z);
 glScalef(size,size,size);
 glCallList(DL);
 glPopMatrix();
}
void Ast::Hit(Ast &a)
{
  float Dx,Dy;
  Dx=a.x-x;
  Dy=a.y-y;

  if(sqrt(Dx*Dx + Dy*Dy) > a.size+size)
	  return;
  if(Dx>Dy){
    a.dx=-a.dx;
	dx=-dx;
  }
  else{
    a.dy=-a.dy;
	dy=-dy;
  }
}

/************************ Bullet STUFF ***********************/
void Bullet::Update(int t)
{
 GameObj::Update(t);
 ttl-=t;
 if(ttl<=0) active=0;
}

void Bullet::Draw(int t)
{
 glPushMatrix();
 glDisable(GL_LIGHTING);
 glTranslatef(x,y,0);
 glRotatef(angle,0,0,1);
 glBegin(GL_LINES);
  glColor3f(.2,1,.6);
  glVertex3f(2.0,0,0);
  glVertex3f(1.0,0,0);
 glEnd();
 glEnable(GL_LIGHTING);
 glPopMatrix();
}
int Bullet::Hit(Ast &A)
{
  float dx,dy;
  dx=A.x-x;
  dy=A.y-y;

  if(dx*dx + dy*dy < A.size*A.size)
	  return 1;
return 0;
}
/************************ Ship STUFF ***********************/

void Ship::Init()
{
 angle=0;
 rotating=0;
 thrust=0;
 dy=0;
 dx=0;
 x=0;
 y=0;
}

void Ship::Update(int t)
{
 int i;
	GameObj::Update(t);

 if(rotating)
		angle=angle+rotating*t*ROTSPEED/1000.0;

 if(thrust)
	{
	 dx=dx+cos(M_PI*angle/180.0)*t*THRUSTACCEL/1000.0;
	 dy=dy+sin(M_PI*angle/180.0)*t*THRUSTACCEL/1000.0;
	}

  for(i=0;i<MAXBULLET;i++)
	  if(B[i].active) B[i].Update(t);
	}

void Ship::ShootBullet(int i)
{
	if(i<0) return;
	B[i].x=x;
	B[i].y=y;
	B[i].dx=BULLETSPEED*cos(M_PI*angle/180.0)/1000.0;
	B[i].dy=BULLETSPEED*sin(M_PI*angle/180.0)/1000.0;
	B[i].angle=angle;
	B[i].active=1;
	B[i].ttl=BULLETTIME;
}

int Ship::GetFreeBullet()
{
	int i;
	for(i=0;i<MAXBULLET;i++)
		if(B[i].active==0)
		return i;
return -1;
}


void Ship::Draw(int t)
{ 
 int i;
 glPushMatrix();
 glTranslatef(x,y,0);
 glRotatef(angle,0,0,1);
 glTranslatef(-3,0,0);
 glRotatef(90,0,1,0);
 glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,redMaterial);
 glutSolidCone(2,6,6,1);
 if(thrust)
 {
	glBegin(GL_LINE_STRIP);
	glVertex3f(-2,-1,0);
	glVertex3f(-4,0,0);
	glVertex3f(-2,1,0);
	glEnd();
 }
 glPopMatrix();

 for(i=0;i<MAXBULLET;i++)
	  if(B[i].active) B[i].Draw(t);
}
/********************* Exp Stuff ***************************/
void Exp::Update(int dt)
{
 GameObj::Update(dt);
 ttl-=dt;
 if(ttl<=0) active=0;
}
void Exp::Draw(int dt)
{
 glPushMatrix();
 glTranslatef(x,y,0);
 glMaterialfv(GL_FRONT,GL_DIFFUSE,yellowMaterial);
 glutSolidSphere(10*(1.0-ttl/1000.0),8,5); 
 glPopMatrix();
}

int GetFreeExp(Exp *E)
{
	int i;
	for(i=0;i<MAXEXP;i++)
		if(E[i].active==0)
		return i;
return -1;
}


/***************************************************************/
/************************ GLUT STUFF ***************************/
/***************************************************************/
static void
  Key(unsigned char key, int x, int y)
{
  switch (key) {
  case 27:
    exit(0);
  case ' ':
	 S.ShootBullet(S.GetFreeBullet());	 
  break;
  }
}

void  SpecialKey(int key, int x, int y)
{
  switch (key) {
  case GLUT_KEY_LEFT: 
	  S.rotating = 1;
	  break;
  case GLUT_KEY_RIGHT: 
	  S.rotating = -1;
	  break;
  case GLUT_KEY_UP: 
	  S.thrust=1;
	  break;
	}
}
void SpecialKeyUp(int key, int x, int y)
{
  switch (key) {
  case GLUT_KEY_LEFT: 
	  S.rotating = 0;
	  break;
  case GLUT_KEY_RIGHT: 
	  S.rotating = 0;
	  break;
  case GLUT_KEY_UP: 
	  S.thrust=0;
	  break;
	}
}

void myGLInit(void)
{
  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_NORMALIZE);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
  glEnable(GL_LIGHT0);
  glEnable(GL_LIGHTING);
  glFlush();
} 

void reshape(GLsizei w, GLsizei h)
{ 
  SW=w;
  SH=h;
  VL = -SPACESIZE;
  VR =  SPACESIZE;
  VB = -SPACESIZE*h/w;
  VT =  SPACESIZE*h/w;
  glViewport(0,0,w,h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(VL,VR,VB,VT,-100,100);
  glMatrixMode(GL_MODELVIEW);
}	

void display(void)
{ 
  static int lasttime;
  int dt;
  int i,j,k;
  dt=glutGet(GLUT_ELAPSED_TIME)-lasttime;
  lasttime=glutGet(GLUT_ELAPSED_TIME);

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glPrintf(5,SH-20,20,"fps %3.1f\n",FPS(dt));
  glutGet(GLUT_ELAPSED_TIME);
  S.Update(dt);
  S.Draw(dt);
  for(i=0;i<MAXAST;i++)
	  if(A[i].active){
		  for(j=0;j<MAXBULLET;j++)
		  if(S.B[j].active)
			  if(S.B[j].Hit(A[i]))
			  {
				  S.B[j].active=0;
				  if(A[i].size>4)
				  {
					  k=GetFreeExp(E);
					  E[k].x=A[i].x;
					  E[k].y=A[i].y;
					  E[k].ttl=1000;
					  E[k].active=1;

					  k=NewAst(A[i].size/2);
					  A[k].x=A[i].x;
					  A[k].y=A[i].y;
					  k=NewAst(A[i].size/2);
					  A[k].x=A[i].x;
					  A[k].y=A[i].y;
				  }
				  A[i].active =0; 				  
			  }
			/*for(j=0;j<MAXAST-i;j++)
			if(A[j].active)
			   A[i].Hit(A[j]);*/

		  A[i].Update(dt);
		  A[i].Draw(dt);
	  }
  for(i=0;i<MAXEXP;i++)
	  if(E[i].active) {
		  E[i].Update(dt);
		  E[i].Draw(dt);
	  }
//	  glFlush();
  glutSwapBuffers();
}

/***************************************************************/
/**************************** MAIN *****************************/
/***************************************************************/

int main(int argc, char** argv)
{ 
	int i;
	
  glutInit(&argc, argv);
  glutInitWindowSize(512, 512);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );
  glutCreateWindow("Asteroids");

  glutReshapeFunc(reshape);
  glutDisplayFunc(display);
  glutIdleFunc(display);
  glutKeyboardFunc(Key);	
  glutSpecialFunc(SpecialKey);	
  glutSpecialUpFunc(SpecialKeyUp);	
    
  myGLInit(); 
  S.Init();  
  for(i=0;i<10;i++)
	  NewAst(5+(rand()%3)*5);
  glutMainLoop();
  return 0;             /* ANSI C requires main to return int. */
}
