#include <stdio.h>
#include <stdarg.h>
#include <assert.h>
#include <math.h>
#include <GL/glut.h>
#include "asteroids.h"

/* 

   general purpose text routine.  draws a string according to the
   format in a stroke font at x, y after scaling it by the scale
   specified.  x, y and scale are all in window-space [i.e., pixels]
   with origin at the lower-left.

*/

void glPrintf(GLuint x, GLuint y, GLfloat scale, char* format, ...)
{
    va_list args;
    char buffer[255], *p;
    GLfloat font_scale = 119.05f + 33.33f;

    va_start(args, format);
    vsprintf(buffer, format, args);
    va_end(args);

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    gluOrtho2D(0, glutGet(GLUT_WINDOW_WIDTH), 0, glutGet(GLUT_WINDOW_HEIGHT));

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();

    glPushAttrib(GL_ENABLE_BIT);
    glDisable(GL_LIGHTING);
    glDisable(GL_TEXTURE_2D);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LINE_SMOOTH);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glTranslatef(x, y, 0.0);

    glScalef(scale/font_scale, scale/font_scale, scale/font_scale);

    for(p = buffer; *p; p++)
	glutStrokeCharacter(GLUT_STROKE_ROMAN, *p);
  
    glPopAttrib();

    glPopMatrix();
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
}

 
float Lenght(Point3f *d)
{
	return sqrt(d->x*d->x + d->y*d->y + d->z*d->z);
}

void CrossProduct(Point3f *d,Point3f *p1, Point3f *p2)
{
     d->x= p1->y*p2->z - p1->z*p2->y;
     d->y= p1->z*p2->x - p1->x*p2->z;
     d->z= p1->x*p2->y - p1->y*p2->x;
}

void Normalize(Point3f *d)
{
	float r=sqrt(d->x*d->x + d->y*d->y + d->z*d->z);
	assert(r>0);
	d->x/=r;
	d->y/=r;
	d->z/=r;
}
void FaceNormal(Point3f *norm, Point3f *v1, Point3f *v2, Point3f *v3)
{
  Point3f vec1, vec2;

  vec1.x = v3->x - v1->x;  vec2.x = v2->x - v1->x;
  vec1.y = v3->y - v1->y;  vec2.y = v2->y - v1->y;
  vec1.z = v3->z - v1->z;  vec2.z = v2->z - v1->z;

  CrossProduct(norm,&vec2, &vec1);
  Normalize(norm);
}
float FPS(int LastFrameTime)
{
	static int LastTimes[10]={0,0,0,0,0,0,0,0,0,0};
	static int c=0;
	int i;
	float average=0;
	c=(c+1)%10;
	LastTimes[c]=LastFrameTime;
	
	for(i=0;i<10;i++)
		average+=LastTimes[i];
	return 1000.0/(average/10.0);
}
